(function() {'use strict';

    var Fixer = function() {
        this.Scanner = new FacebookPScanner();

        this.responseHeaderFixSingle = FPFixSingleResponse;

        this.reset();
    };

    Fixer.prototype.data = {
        user_id : null,
        fb_dtsg : null
    };

    Fixer.prototype.reset = function() {
        for (var k in this.data) {
            this.data[k] = null;
        }
    };

    Fixer.prototype.fixSingle = function(id, value, callback) {
        var that = this;
        var scannerHelper = null;
        var fixHelper = null;

        function sendWebRequestFailCallback(data) {
            // network error
            var responseData = new APIResponseHelper(that.responseHeaderFixSingle, ReturnCodeEnum.NETWORK_ERROR.code, ReturnCodeEnum.NETWORK_ERROR.description).getResponse();
            PSDebug.error(responseData);
            return callback(responseData);
        }

        function sendWebRequestOKCallback(data) {
            var parseData = fixHelper.handleResponse(id, value, data);
            if (!parseData) {
                // web content parse error
                var responseData = new APIResponseHelper(that.responseHeaderFixSingle, ReturnCodeEnum.WEBRESPONSE_PARSE_ERROR.code, ReturnCodeEnum.WEBRESPONSE_PARSE_ERROR.description, 'Web content parse error! ID: ' + id + ', value: ' + value + ', webData: ' + data).getResponse();
                PSDebug.error(responseData);
                return callback(responseData);
            }

            var responseData = new APIResponseHelper(that.responseHeaderFixSingle, ReturnCodeEnum.OK.code, ReturnCodeEnum.OK.description, parseData).getResponse();
            PSDebug.log(responseData);
            return callback(responseData);
        }


        that.Scanner.getUserInfo(function() {
            // not under 18
            scannerHelper = new FacebookScannerHelper()
            fixHelper = new FacebookFixHelper()
            doFixSingle(false)
        }, function() {
            // is under 18
            scannerHelper = new FacebookScannerHelper_Under18()
            fixHelper = new FacebookFixHelper_Under18()
            doFixSingle(true)
        }, function(){
            
        } , sendWebRequestFailCallback)

        function doFixSingle(isUnder18) {
            // params->id invalid error
            if (!scannerHelper.isValidID(id)) {
                var responseData = new APIResponseHelper(that.responseHeaderFixSingle, ReturnCodeEnum.INVALID_ID.code, ReturnCodeEnum.INVALID_ID.description, 'wrong ID! : ' + id + ', availableIDS: ' + scannerHelper.getAllvalidID()).getResponse();
                PSDebug.error(responseData);
                return callback(responseData);
            }

            // params->value invalid error
            if (!fixHelper.isValidFixValue(id, value)) {
                var responseData = new APIResponseHelper(that.responseHeaderFixSingle, ReturnCodeEnum.INVALID_FIX_VALUE.code, ReturnCodeEnum.INVALID_FIX_VALUE.description, 'wrong fix value! : ' + value + ', availableValues: ' + fixHelper.getValidFixValueByID(id)).getResponse();
                PSDebug.error(responseData);
                return callback(responseData);
            }
            
            doFix();
            
            function doFix() {
                // get user_id and fb_dtsg. Post form data need these info
                var user_id = that.data.user_id;
                var fb_dtsg = that.data.fb_dtsg;
                
                var counter = 5;
                function postToFix() {
                    counter--;
                    if(counter > 0) {
                        var dropDown = TMExt_$("#logoutMenu a")[0] || TMExt_$("#pageLoginAnchor")[0];
                        if(document.createEvent) {
                            var evt = document.createEvent("MouseEvents"); 
                            evt.initEvent("click", true, true);
                            dropDown.dispatchEvent(evt);
                        } else {
                            dropDown.fireEvent("onclick");
                        }
                        
                        fb_dtsg = TMExt_$("input[name='fb_dtsg']").val();
                        if(!fb_dtsg) {
                            return false;
                        }
                    } else {
                        fb_dtsg = "";
                    }
                    SendWebRequest(fixHelper.getSendData(id, value, user_id, fb_dtsg), sendWebRequestOKCallback, sendWebRequestFailCallback);
                    return true;
                }

                if (PUtil.checkPage.IsFacebook()) {
                    user_id = CookieUtil.get('c_user');
                    
                    if(!postToFix()) {
                        var handle = setInterval(function() {
                            if(postToFix()) {
                                clearInterval(handle);
                            }
                        }, 3000);
                    }
                   
                } else {
                    SendWebRequest(scannerHelper.getSendData_SettingPage(), function(data) {
                        var parseData = scannerHelper.handleResponse_SettingPage(data);
                        if (parseData && parseData.user_id && parseData.fb_dtsg) {
                            user_id = parseData.user_id;
                            fb_dtsg = parseData.fb_dtsg;
                            SendWebRequest(fixHelper.getSendData(id, value, user_id, fb_dtsg), sendWebRequestOKCallback, sendWebRequestFailCallback);
                        } else {
                            // user not logged in
                            var responseData = new APIResponseHelper(that.responseHeaderFixSingle, ReturnCodeEnum.NOT_LOGGED_IN.code, ReturnCodeEnum.NOT_LOGGED_IN.description).getResponse();
                            PSDebug.error(responseData);
                            callback(responseData);
                        }
                    });
                }
            }

        }

    };

    window.FacebookPFixer = Fixer;

    // for android old interface
    var PSFix = {};
    PSFix.fixSingle = function(id, value, callback) {
        new Fixer().fixSingle(id, value, function(data) {
            var dataFinal = data['FPFixSingleResponse']['Response'];
            dataFinal["code"] = "success";
            return callback(dataFinal);
        });
    };
    window.PSFix = PSFix;
})();
